@@ -49,6 +49,7 @@
 #include <QPushButton>
 #include <QRadioButton>
 #include <QScrollArea>
+#include <QToolTip>
 #include <QVBoxLayout>
 
 #include <QGpgME/DefaultKeyGenerationJob>
@@ -69,6 +70,7 @@ public:
     OpenPGPFilter() : DefaultKeyFilter()
     {
         setIsOpenPGP(DefaultKeyFilter::Set);
+        setCanEncrypt(DefaultKeyFilter::Set);
     }
 };
 static std::shared_ptr<KeyFilter> s_pgpFilter = std::shared_ptr<KeyFilter> (new OpenPGPFilter);
@@ -78,6 +80,10 @@ public:
     OpenPGPSignFilter() : DefaultKeyFilter()
     {
         /* Also list unusable keys to make it transparent why they are unusable */
+        setDisabled(DefaultKeyFilter::NotSet);
+        setRevoked(DefaultKeyFilter::NotSet);
+        setExpired(DefaultKeyFilter::NotSet);
+        setCanSign(DefaultKeyFilter::Set);
         setHasSecret(DefaultKeyFilter::Set);
         setIsOpenPGP(DefaultKeyFilter::Set);
     }
@@ -89,6 +95,7 @@ public:
     SMIMEFilter(): DefaultKeyFilter()
     {
         setIsOpenPGP(DefaultKeyFilter::NotSet);
+        setCanEncrypt(DefaultKeyFilter::Set);
     }
 };
 static std::shared_ptr<KeyFilter> s_smimeFilter = std::shared_ptr<KeyFilter> (new SMIMEFilter);
@@ -97,6 +104,10 @@ class SMIMESignFilter: public DefaultKeyFilter
 public:
     SMIMESignFilter(): DefaultKeyFilter()
     {
+        setDisabled(DefaultKeyFilter::NotSet);
+        setRevoked(DefaultKeyFilter::NotSet);
+        setExpired(DefaultKeyFilter::NotSet);
+        setCanSign(DefaultKeyFilter::Set);
         setIsOpenPGP(DefaultKeyFilter::NotSet);
         setHasSecret(DefaultKeyFilter::Set);
     }
@@ -120,12 +131,23 @@ class ComboWidget: public QWidget
 public:
     explicit ComboWidget(KeySelectionCombo *combo):
         mCombo(combo),
-        mFilterBtn(new QPushButton)
+        mFilterBtn(new QPushButton),
+        mFromOverride(GpgME::UnknownProtocol)
     {
         auto hLay = new QHBoxLayout(this);
+        auto infoBtn = new QPushButton;
+        infoBtn->setIcon(QIcon::fromTheme(QStringLiteral("help-contextual")));
+        infoBtn->setIconSize(QSize(22,22));
+        infoBtn->setFlat(true);
+        hLay->addWidget(infoBtn);
         hLay->addWidget(combo, 1);
         hLay->addWidget(mFilterBtn, 0);
 
+        connect(infoBtn, &QPushButton::clicked, this, [this, infoBtn] () {
+            QToolTip::showText(infoBtn->mapToGlobal(QPoint()) + QPoint(infoBtn->width(), 0),
+                    mCombo->currentData(Qt::ToolTipRole).toString(), infoBtn, QRect(), 30000);
+        });
+
         // Assume that combos start out with a filter
         mFilterBtn->setIcon(QIcon::fromTheme(QStringLiteral("kt-remove-filters")));
         mFilterBtn->setToolTip(i18n("Remove Filter"));
@@ -156,10 +178,21 @@ public:
         return mCombo;
     }
 
+    GpgME::Protocol fromOverride() const
+    {
+        return mFromOverride;
+    }
+
+    void setFromOverride(GpgME::Protocol proto)
+    {
+        mFromOverride = proto;
+    }
+
 private:
     KeySelectionCombo *mCombo;
     QPushButton *mFilterBtn;
     QString mLastIdFilter;
+    GpgME::Protocol mFromOverride;
 };
 
 static enum GpgME::UserID::Validity keyValidity(const GpgME::Key &key)
@@ -175,6 +208,17 @@ static enum GpgME::UserID::Validity keyValidity(const GpgME::Key &key)
 
     return validity;
 }
+
+static bool key_has_addr(const GpgME::Key &key, const QString &addr)
+{
+    for (const auto &uid: key.userIDs()) {
+        if (QString::fromStdString(uid.addrSpec()).toLower() == addr.toLower()) {
+            return true;
+        }
+    }
+    return false;
+}
+
 } // namespace
 
 class NewKeyApprovalDialog::Private
@@ -195,6 +239,13 @@ public:
         mAllowMixed(allowMixed),
         q(pub)
     {
+        // We do the translation here to avoid having the same string multiple times.
+        mGenerateTooltip = i18nc("@info:tooltip for a 'Generate new key pair' action "
+                                 "in a combobox when a user does not yet have an OpenPGP or S/MIME key.",
+                                 "Generate a new key using your E-Mail address.<br/><br/>"
+                                 "The key is necessary to decrypt and sign E-Mails. "
+                                 "You will be asked for a passphrase to protect this key and the protected key "
+                                 "will be stored in your home directory.");
         mMainLay = new QVBoxLayout;
 
         QDialogButtonBox *btnBox = new QDialogButtonBox(QDialogButtonBox::Ok | QDialogButtonBox::Cancel);
@@ -213,7 +264,7 @@ public:
         mScrollArea->setFrameStyle(QFrame::NoFrame);
         mScrollLayout->setContentsMargins(0, 0, 0, 0);
 
-        q->setWindowTitle(i18n("Security approval"));
+        q->setWindowTitle(i18nc("@title:window", "Security approval"));
 
         auto fmtLayout = new QHBoxLayout;
         mFormatBtns = new QButtonGroup;
@@ -249,8 +300,8 @@ public:
         updateFilter();
 
 
-        QObject::connect (mFormatBtns, static_cast<void (QButtonGroup::*)(int, bool)> (&QButtonGroup::buttonToggled),
-                q, [this](int, bool) {
+        QObject::connect (mFormatBtns, static_cast<void (QButtonGroup::*)(QAbstractButton *, bool)> (&QButtonGroup::buttonToggled),
+                q, [this](QAbstractButton *, bool) {
             updateFilter();
         });
 
@@ -275,7 +326,7 @@ public:
         auto progress = new Kleo::ProgressDialog(job, i18n("Generating key for '%1'...", addr) + QStringLiteral("\n\n") +
                                                  i18n("This can take several minutes."), q);
         progress->setWindowFlags(progress->windowFlags() & ~Qt::WindowContextHelpButtonHint);
-        progress->setWindowTitle(i18n("Key generation"));
+        progress->setWindowTitle(i18nc("@title:window", "Key generation"));
         progress->setModal(true);
         progress->setAutoClose(true);
         progress->setMinimumDuration(0);
@@ -294,7 +345,7 @@ public:
     {
         mLastError = result.error();
         if (!mLastError || mLastError.isCanceled()) {
-            combo->setDefaultKey(QString::fromLatin1(result.fingerprint()));
+            combo->setDefaultKey(QString::fromLatin1(result.fingerprint()), GpgME::OpenPGP);
             connect (combo, &KeySelectionCombo::keyListingFinished, q, [this, job] () {
                     mRunningJobs.removeAll(job);
                 });
@@ -311,9 +362,52 @@ public:
             mRunningJobs.clear();
             return;
         }
-        if (mRunningJobs.empty()) {
-            q->accept();
+
+        if (!mRunningJobs.empty()) {
+            return;
         }
+        /* Save the keys */
+        bool isPGP = mFormatBtns->checkedId() == 1;
+        bool isSMIME = mFormatBtns->checkedId() == 2;
+
+        mAcceptedEnc.clear();
+        mAcceptedSig.clear();
+
+        for (const auto combo: qAsConst(mEncCombos)) {
+            const auto &addr = combo->property("address").toString();
+            const auto &key = combo->currentKey();
+            if (!combo->isVisible()) {
+                continue;
+            }
+            if (isSMIME && key.protocol() != GpgME::CMS) {
+                continue;
+            }
+            if (isPGP && key.protocol() != GpgME::OpenPGP) {
+                continue;
+            }
+            if (mAcceptedEnc.contains(addr)) {
+                mAcceptedEnc[addr].push_back(key);
+            } else {
+                std::vector<GpgME::Key> vec;
+                vec.push_back(key);
+                mAcceptedEnc.insert(addr, vec);
+            }
+        }
+        for (const auto combo: qAsConst(mSigningCombos)) {
+            const auto key = combo->currentKey();
+            if (!combo->isVisible()) {
+                continue;
+            }
+            if (isSMIME && key.protocol() != GpgME::CMS) {
+                continue;
+            }
+            if (isPGP && key.protocol() != GpgME::OpenPGP) {
+                continue;
+            }
+            mAcceptedSig.push_back(combo->currentKey());
+        }
+
+        q->accept();
     }
 
     void accepted()
@@ -321,7 +415,7 @@ public:
         // We can assume everything was validly resolved, otherwise
         // the OK button would have been disabled.
         // Handle custom items now.
-        for (auto combo: mAllCombos) {
+        for (auto combo: qAsConst(mAllCombos)) {
             auto act = combo->currentData(Qt::UserRole).toInt();
             if (act == GenerateKey) {
                 generateKey(combo);
@@ -347,11 +441,28 @@ public:
             mCurEncFilter = s_defaultFilter;
             mCurSigFilter = s_signFilter;
         }
-        for (auto combo: mSigningCombos) {
+        for (auto combo: qAsConst(mSigningCombos)) {
             combo->setKeyFilter(mCurSigFilter);
+            auto widget = qobject_cast <ComboWidget *>(combo->parentWidget());
+            if (!widget) {
+                qCDebug(LIBKLEO_LOG) << "Failed to find signature combo widget";
+                continue;
+            }
+            widget->setVisible(widget->fromOverride() == GpgME::UnknownProtocol ||
+                               ((isSMIME && widget->fromOverride() == GpgME::CMS) ||
+                                (isPGP && widget->fromOverride() == GpgME::OpenPGP)));
         }
-        for (auto combo: mEncCombos) {
+        for (auto combo: qAsConst(mEncCombos)) {
             combo->setKeyFilter(mCurEncFilter);
+
+            auto widget = qobject_cast <ComboWidget *>(combo->parentWidget());
+            if (!widget) {
+                qCDebug(LIBKLEO_LOG) << "Failed to find combo widget";
+                continue;
+            }
+            widget->setVisible(widget->fromOverride() == GpgME::UnknownProtocol ||
+                               ((isSMIME && widget->fromOverride() == GpgME::CMS) ||
+                                (isPGP && widget->fromOverride() == GpgME::OpenPGP)));
         }
     }
 
@@ -360,14 +471,17 @@ public:
         auto combo = new KeySelectionCombo();
         combo->setKeyFilter(mCurSigFilter);
         if (!key.isNull()) {
-            combo->setDefaultKey(QString::fromLatin1(key.primaryFingerprint()));
+            combo->setDefaultKey(QString::fromLatin1(key.primaryFingerprint()), key.protocol());
         }
         if (key.isNull() && mProto != GpgME::CMS) {
             combo->appendCustomItem(QIcon::fromTheme(QStringLiteral("document-new")),
-                                    i18n("Generate a new key pair"), GenerateKey);
+                                    i18n("Generate a new key pair"), GenerateKey,
+                                    mGenerateTooltip);
         }
         combo->appendCustomItem(QIcon::fromTheme(QStringLiteral("emblem-unavailable")),
-                i18n("Don't confirm identity and integrity"), IgnoreKey);
+                i18n("Don't confirm identity and integrity"), IgnoreKey,
+                i18nc("@info:tooltip for not selecting a key for signing.",
+                      "The E-Mail will not be cryptographically signed."));
 
         mSigningCombos << combo;
         mAllCombos << combo;
@@ -376,7 +490,11 @@ public:
         connect(combo, &KeySelectionCombo::currentKeyChanged, q, [this] () {
             updateOkButton();
         });
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
         connect(combo, QOverload<int>::of(&QComboBox::currentIndexChanged), q, [this] () {
+#else
+        connect(combo, QOverload<int, const QString &>::of(&QComboBox::currentIndexChanged), q, [this] () {
+#endif
             updateOkButton();
         });
 
@@ -400,13 +518,16 @@ public:
             for (const auto &key: resolved[addr])
             {
                 auto comboWidget = createSigningCombo(addr, key);
-                if (resolved[addr].size() == 1) {
+                if (key_has_addr (key, addr)) {
                     comboWidget->combo()->setIdFilter(addr);
                 }
+                if (resolved[addr].size() > 1) {
+                    comboWidget->setFromOverride(key.protocol());
+                }
                 sigLayout->addWidget(comboWidget);
             }
         }
-        for (const QString &addr: unresolved) {
+        for (const QString &addr: qAsConst(unresolved)) {
             auto group = new QGroupBox(i18nc("Caption for signing key selection, no key found",
                                              "No key found for the address '%1':", addr));
             group->setAlignment(Qt::AlignLeft);
@@ -430,25 +551,35 @@ public:
             auto combo = new KeySelectionCombo(false);
             combo->setKeyFilter(mCurEncFilter);
             if (!key.isNull()) {
-                combo->setDefaultKey(QString::fromLatin1(key.primaryFingerprint()));
+                combo->setDefaultKey(QString::fromLatin1(key.primaryFingerprint()),
+                                     key.protocol());
             }
 
             if (mSender == addr && key.isNull()) {
                 combo->appendCustomItem(QIcon::fromTheme(QStringLiteral("document-new")),
-                                        i18n("Generate a new key pair"), GenerateKey);
+                                        i18n("Generate a new key pair"), GenerateKey,
+                                        mGenerateTooltip);
             }
 
             combo->appendCustomItem(QIcon::fromTheme(QStringLiteral("emblem-unavailable")),
-                    i18n("Ignore recipient"), IgnoreKey);
+                    i18n("No key. Recipient will be unable to decrypt."), IgnoreKey,
+                    i18nc("@info:tooltip for No Key selected for a specific recipient.",
+                          "Do not select a key for this recipient.<br/><br/>"
+                          "The recipient will receive the encrypted E-Mail, but it can only "
+                          "be decrypted with the other keys selected in this dialog."));
 
-            if (keys.size () == 1) {
+            if (key.isNull() || key_has_addr (key, addr)) {
                 combo->setIdFilter(addr);
             }
 
             connect(combo, &KeySelectionCombo::currentKeyChanged, q, [this] () {
                 updateOkButton();
             });
+#if QT_VERSION < QT_VERSION_CHECK(5, 15, 0)
             connect(combo, QOverload<int>::of(&QComboBox::currentIndexChanged), q, [this] () {
+#else
+            connect(combo, QOverload<int, const QString &>::of(&QComboBox::currentIndexChanged), q, [this] () {
+#endif
                 updateOkButton();
             });
 
@@ -456,6 +587,9 @@ public:
             mAllCombos << combo;
             combo->setProperty("address", addr);
             auto comboWidget = new ComboWidget(combo);
+            if (keys.size() > 1) {
+                comboWidget->setFromOverride(key.protocol());
+            }
             encGrid->addWidget(comboWidget, encGrid->rowCount(), 0, 1, 2);
         }
     }
@@ -508,7 +642,7 @@ public:
             mOkButton->setText(origOkText);
         }
 
-        if (Formatting::complianceMode() != QStringLiteral("de-vs")) {
+        if (Formatting::complianceMode() != QLatin1String("de-vs")) {
             return;
         }
 
@@ -580,6 +714,9 @@ public:
     QList <QGpgME::Job *> mRunningJobs;
     GpgME::Error mLastError;
     QLabel *mComplianceLbl;
+    QMap<QString, std::vector<GpgME::Key> > mAcceptedEnc;
+    std::vector<GpgME::Key> mAcceptedSig;
+    QString mGenerateTooltip;
 };
 
 NewKeyApprovalDialog::NewKeyApprovalDialog(const QMap<QString, std::vector<GpgME::Key> > &resolvedSigningKeys,
@@ -596,6 +733,7 @@ NewKeyApprovalDialog::NewKeyApprovalDialog(const QMap<QString, std::vector<GpgME
 {
     d->addSigningKeys(resolvedSigningKeys, unresolvedSigKeys);
     d->addEncryptionKeys(resolvedRecp, unresolvedRecp);
+    d->updateFilter();
     d->updateOkButton();
 
     const auto size = sizeHint();
@@ -605,30 +743,12 @@ NewKeyApprovalDialog::NewKeyApprovalDialog(const QMap<QString, std::vector<GpgME
 
 std::vector<GpgME::Key> NewKeyApprovalDialog::signingKeys()
 {
-    std::vector <GpgME::Key> ret;
-
-    for (const auto combo: d->mSigningCombos) {
-        ret.push_back(combo->currentKey());
-    }
-
-    return ret;
+    return d->mAcceptedSig;
 }
 
 QMap <QString, std::vector<GpgME::Key> > NewKeyApprovalDialog::encryptionKeys()
 {
-    QMap <QString, std::vector<GpgME::Key> > ret;
-    for (const auto combo: d->mEncCombos) {
-        const auto &addr = combo->property("address").toString();
-        const auto &key = combo->currentKey();
-        if (ret.contains(addr)) {
-            ret[addr].push_back(key);
-        } else {
-            std::vector<GpgME::Key> vec;
-            vec.push_back(key);
-            ret.insert(addr, vec);
-        }
-    }
-    return ret;
+    return d->mAcceptedEnc;
 }
 
 #include "newkeyapprovaldialog.moc"
